(function () {
  'use strict';
  // Source: src/html/scripts/admin/travel-edit.js
  angular.module('tour-edit', []).controller('TourEditCtrl', [
    '$scope',
    '$modal',
    function ($scope, $modal) {
      $scope.title = '{{copy.title}}';
      var copy = $scope.copy;
      $scope.edit = {};
      if (!copy.photos) {
        copy.photos = [];
      }
      function addDestination(d) {
        if (!copy.destinations || !angular.isArray(copy.destinations)) {
          copy.destinations = [];
        }
        if (copy.destinations.indexOf(d) === -1)
          copy.destinations.push(d);
        delete $scope.edit.destination;
      }
      function addPhoto(photo) {
        copy.photos.push(photo);
      }
      function removePhoto(index) {
        copy.photos.splice(index, 1);
      }
      function editPhoto(index) {
        promptPhoto(copy.photos[index]);
      }
      function promptPhoto(photo) {
        var modalInstance = $modal.open({
            templateUrl: 'photoPrompDialog.html',
            controller: 'PhotoEditCtrl',
            scope: $scope,
            resolve: {
              photos: function () {
                return copy.photos;
              },
              photo: function () {
                return photo;
              }
            }
          });
        modalInstance.result.then(addPhoto);
      }
      function updateRates(rate) {
        if (!copy.rates)
          copy.rates = [];
        if (rate.original && copy.rates.indexOf(rate.original) > -1) {
          var i = copy.rates.indexOf(rate.original);
          angular.extend(copy.rates[i], rate.copy);
        } else {
          copy.rates.push(rate.copy);
        }
      }
      function manageRates(entry) {
        var modalInstance = $modal.open({
            templateUrl: 'edit-rate-dialog',
            controller: 'RateEditCtrl',
            size: 'lg',
            resolve: {
              rate: function () {
                if (entry) {
                  return {
                    original: entry,
                    copy: angular.copy(entry)
                  };
                }
                return {
                  original: null,
                  copy: null
                };
              }
            }
          });
        modalInstance.result.then(updateRates);
      }
      $scope.promptPhoto = promptPhoto;
      $scope.editPhoto = editPhoto;
      $scope.removePhoto = removePhoto;
      $scope.manageRates = manageRates;
      $scope.addDestination = addDestination;
    }
  ]).controller('PhotoEditCtrl', [
    '$scope',
    '$modalInstance',
    'photos',
    'photo',
    function ($scope, $modalInstance, photos, photo) {
      // TODO do not allow duplicate photos.
      if (photo)
        $scope.photo = photo;
      $scope.ok = function (photo) {
        console.log(photo);
        $modalInstance.close(photo);
      };
      $scope.cancel = function () {
        $modalInstance.dismiss('cancel');
      };
    }
  ]).controller('RateEditCtrl', [
    '$scope',
    '$modalInstance',
    'rate',
    function ($scope, $modalInstance, rate) {
      // TODO do not allow duplicate photos.
      var week = [
          'Lun',
          'Mar',
          'Mer',
          'Jeu',
          'Ven',
          'Sam',
          'Dim'
        ];
      $scope.rate = rate;
      rate.copy = rate.copy || {
        type: 'dates',
        week: {},
        dates: [],
        rates: {
          'std': 0,
          'single': 0,
          'child': 0,
          'baby': 0
        }
      };
      $scope.selection = rate.copy;
      $scope.rateSelectionTypes = [
        'dates',
        'period'
      ];
      $scope.rateLabels = {
        'std': 'Demi Double',
        'single': 'Single',
        'child': 'Enfant',
        'baby': 'B\xe9b\xe9'
      };
      $scope.options = { week: week };
      $scope.dateToAdd = {};
      $scope.addDate = function () {
        $scope.selection.dates.push($scope.dateToAdd.value);
        delete $scope.dateToAdd.value;
        console.log($scope.selection.dates);
      };
      $scope.toggleWeekDay = function (day) {
        $scope.selection.week[day] = !$scope.selection.week[day];
      };
      $scope.selectionType = function (t) {
        if (t === $scope.selection.type)
          return;
        $scope.selection.type = t;
      };
      $scope.ok = function () {
        $modalInstance.close(rate);
      };
      $scope.cancel = function () {
        $modalInstance.dismiss('cancel');
      };  //$scope.selectionType('dates');
    }
  ]);
}());